package com.nanyan.securitylink.filter;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONObject;
import com.nanyan.securitylink.service.AlertService;
import com.nanyan.securitylink.service.TokenRouteMappingService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.factory.AbstractGatewayFilterFactory;
import org.springframework.core.io.buffer.DataBuffer;
import org.springframework.core.io.buffer.DataBufferUtils;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpMethod;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.reactive.function.BodyInserters;
import org.springframework.web.reactive.function.client.WebClient;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;
import java.nio.charset.StandardCharsets;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

@Slf4j
@Component
public class FailoverGatewayFilterFactory extends AbstractGatewayFilterFactory<FailoverGatewayFilterFactory.Config> {

    private static final ThreadLocal<String> REQUEST_BODY_CACHE = new ThreadLocal<>();
    private final WebClient.Builder webClientBuilder;
    private final TokenRouteMappingService tokenRouteMappingService;
    // 添加错误统计
    private static final int WINDOW_SIZE_SECONDS = 10*60; // 1分钟窗口
    private static final double ERROR_THRESHOLD = 0.5; // 75%错误率阈值
    @Autowired
    AlertService alertService;
    private final Map<String, CircularQueue> errorStatsMap = new ConcurrentHashMap<>();

    public FailoverGatewayFilterFactory(WebClient.Builder webClientBuilder,
                                        TokenRouteMappingService tokenRouteMappingService) {
        super(Config.class);
        this.webClientBuilder = webClientBuilder;
        this.tokenRouteMappingService = tokenRouteMappingService;
    }

    // 内部类：环形队列，用于记录请求结果
    private static class CircularQueue {
        private final long[] timestamps;
        private final boolean[] results; // true表示成功，false表示失败
        private int currentIndex = 0;
        private final int capacity;
        private final Object lock = new Object();

        public CircularQueue(int capacity) {
            this.capacity = capacity;
            this.timestamps = new long[capacity];
            this.results = new boolean[capacity];
        }

        public void add(boolean success) {
            synchronized (lock) {
                timestamps[currentIndex] = System.currentTimeMillis();
                results[currentIndex] = success;
                currentIndex = (currentIndex + 1) % capacity;
            }
        }

        public double getErrorRate() {
            synchronized (lock) {
                long currentTime = System.currentTimeMillis();
                long oneMinuteAgo = currentTime - WINDOW_SIZE_SECONDS * 1000;
                int totalRequests = 0;
                int failedRequests = 0;

                for (int i = 0; i < capacity; i++) {
                    if (timestamps[i] > oneMinuteAgo) {
                        totalRequests++;
                        if (!results[i]) {
                            failedRequests++;
                        }
                    }
                }

                return totalRequests == 0 ? 0 : (double) failedRequests / totalRequests;
            }
        }
    }

    private CircularQueue getOrCreateErrorStats(String primaryUrl) {
        return errorStatsMap.computeIfAbsent(primaryUrl,
                k -> new CircularQueue(WINDOW_SIZE_SECONDS * 10)); // 存储10次/秒的采样
    }

    private boolean shouldSkipPrimary(String primaryUrl) {
        CircularQueue stats = getOrCreateErrorStats(primaryUrl);
        double errorRate = stats.getErrorRate();
        log.info("rate: {}%, url: {}",errorRate * 100, primaryUrl);
        boolean skip = errorRate >= ERROR_THRESHOLD;
        if (skip) {
            // 发送告警
            sendAlarmIfNeeded(primaryUrl, errorRate);
            log.warn("Skipping primary endpoint due to high error rate: {}%, url: {}",
                    errorRate * 100, primaryUrl);
        }
        return skip;
    }
    private final Map<String, Long> alarmSentMap = new ConcurrentHashMap<>();
    private void sendAlarmIfNeeded(String primaryUrl, double errorRate) {
        long currentTime = System.currentTimeMillis();
        Long lastAlarmTime = alarmSentMap.get(primaryUrl);

        // 如果从未发送过告警，或者距离上次告警已经超过冷却时间
        if (lastAlarmTime == null || (currentTime - lastAlarmTime) > WINDOW_SIZE_SECONDS*1000) {
            String alarmMessage = String.format(
                    "Service degradation detected: %s has high error rate (%.2f%%), switching to fallback service",
                    primaryUrl, errorRate * 100);

            try {
                alertService.sendAlertMsg("dify 异常，已经切换到JAVA服务");
                alarmSentMap.put(primaryUrl, currentTime);
                log.warn("Alarm sent: {}", alarmMessage);
            } catch (Exception e) {
                log.error("Failed to send alarm: {}", e.getMessage());
            }
        }
    }

    @Override
    public GatewayFilter apply(Config config) {
        return (exchange, chain) -> {
            try {
                String authHeader = exchange.getRequest().getHeaders().getFirst(HttpHeaders.AUTHORIZATION);
                String token = authHeader != null && authHeader.startsWith("Bearer ")
                        ? authHeader.substring(7)
                        : null;
                String sourceUri = exchange.getRequest().getPath().value();

                if (token != null) {
                    return tokenRouteMappingService.findMatchingUriConfig(token, sourceUri)
                            .map(uriConfig -> {
                                return cacheRequestBody(exchange).flatMap(cachedBody -> {
                                    log.info("Found matching URI config for token: {}, sourceUri: {}", token, sourceUri);
                                    String primaryUrl = uriConfig.getPrimaryUrl();
                                    CircularQueue errorStats = getOrCreateErrorStats(primaryUrl);
                                    // 检查是否应该跳过主地址
                                    if (shouldSkipPrimary(uriConfig.getPrimaryUrl())) {
                                        //告警日志
                                        log.info("Directly using fallback due to high error rate");
                                        String targetUri = tokenRouteMappingService.resolveTargetUri(sourceUri, uriConfig);
                                        return tryRequest(
                                                uriConfig.getFallbackUrl(),
                                                targetUri,
                                                uriConfig.getFallbackHost(),
                                                uriConfig.getModel(),
                                                uriConfig.getModelKey(),
                                                exchange,
                                                false,
                                                null
                                        );
                                    }

                                    return tryRequest(
                                            uriConfig.getPrimaryUrl(),
                                            sourceUri,
                                            uriConfig.getPrimaryHost(),
                                            uriConfig.getModel(),
                                            uriConfig.getModelKey(),
                                            exchange,
                                            true,
                                            errorStats
                                    ).onErrorResume(primaryError -> {
                                        log.error("Primary endpoint failed: {}, error: {}",
                                                uriConfig.getPrimaryUrl() + sourceUri,
                                                primaryError.getMessage());
                                        // 记录错误
                                        getOrCreateErrorStats(uriConfig.getPrimaryUrl()).add(false);

                                        String targetUri = tokenRouteMappingService.resolveTargetUri(sourceUri, uriConfig);
                                        return tryRequest(
                                                uriConfig.getFallbackUrl(),
                                                targetUri,
                                                uriConfig.getFallbackHost(),
                                                uriConfig.getModel(),
                                                uriConfig.getModelKey(),
                                                exchange,
                                                false,
                                                null
                                        );
                                    });
                                });
                            })
                            .orElse(chain.filter(exchange));
                }

                return chain.filter(exchange);
            } finally {
                // 确保在请求处理完成后清理 ThreadLocal
                REQUEST_BODY_CACHE.remove();
            }
        };
    }

    private Mono<String> cacheRequestBody(ServerWebExchange exchange) {

        ServerHttpRequest request = exchange.getRequest();
        if (!requiresRequestBody(request.getMethod())) {
            return Mono.just("");
        }

        // 读取并缓存请求体
        return DataBufferUtils.join(request.getBody())
                .map(dataBuffer -> {
                    try {
                        byte[] bytes = new byte[dataBuffer.readableByteCount()];
                        dataBuffer.read(bytes);
                        String body = new String(bytes, StandardCharsets.UTF_8);
                        // 将请求体存储在 ThreadLocal 中
                        log.info("cache body:{}", body);
                        REQUEST_BODY_CACHE.set(body);
                        return body;
                    } finally {
                        DataBufferUtils.release(dataBuffer);
                    }
                })
                .defaultIfEmpty("");
    }

    private Mono<Void> tryRequest(String baseUrl, String uri, String host, String model, String modelKey,
                                  ServerWebExchange exchange, boolean isSourceRequest, CircularQueue errorStats) {
        String cachedBody = REQUEST_BODY_CACHE.get();
        ServerHttpRequest request = exchange.getRequest();
        String fullUrl = baseUrl + (baseUrl.endsWith("/") ? uri.substring(1) : uri);

        log.info("Trying request to: {}", fullUrl);
        log.info("Using cached request body: {}", cachedBody);

        WebClient.RequestBodySpec requestBodySpec = webClientBuilder.build()
                .method(request.getMethod())
                .uri(fullUrl + (request.getQueryParams().isEmpty() ? "" :
                        "?" + request.getQueryParams().toString().substring(1)));

        // 复制请求头
        requestBodySpec.headers(headers -> {
            headers.addAll(request.getHeaders());
            headers.set(HttpHeaders.HOST, host);
            headers.set("model", model);
            headers.set("modelKey", modelKey);
        });

        // 设置请求体
        if (requiresRequestBody(request.getMethod()) && cachedBody != null && !cachedBody.isEmpty()) {
            MediaType contentType = request.getHeaders().getContentType();
            if (contentType != null) {
                requestBodySpec.contentType(contentType);
            }
            requestBodySpec.body(BodyInserters.fromValue(cachedBody));
        }

        return requestBodySpec
                .exchange()
                .flatMap(clientResponse -> handleResponse(exchange, clientResponse, isSourceRequest, errorStats));
    }

    private Mono<Void> handleResponse(ServerWebExchange exchange,
                                      org.springframework.web.reactive.function.client.ClientResponse clientResponse,
                                      boolean isSourceRequest,CircularQueue errorStats) {
        HttpStatus responseStatus = clientResponse.statusCode();

        if (isSourceRequest) {
            return clientResponse.bodyToMono(String.class)
                    .flatMap(body -> {
                        log.info("Source request response: status={}, body={}", responseStatus, body);

                        if (responseStatus != HttpStatus.OK) {
                            // 只在 source 请求时记录错误
                            if (errorStats != null) {
                                errorStats.add(false);
                            }
                            return Mono.error(new RuntimeException(
                                    String.format("Source request HTTP status not 200: %s, body: %s",
                                            responseStatus.value(), body)));
                        }

                        try {
                            JSONObject jsonBody = JSON.parseObject(body);
                            boolean success = false;
                            String bodyStatus = "";
                            if(jsonBody.containsKey("status")){
                                Integer status = jsonBody.getInteger("status");
                                bodyStatus = status +"";
                            }
                            if(jsonBody.containsKey("answer")){
                                success = true;
                            }

                            if(jsonBody.containsKey("data")){
                                JSONObject data = jsonBody.getJSONObject("data");
                                if (data.containsKey("status")){
                                    String status = data.getString("status");
                                    if("succeeded".equalsIgnoreCase(status)){
                                        success = true;
                                    }
                                    bodyStatus = status;
                                }
                            }
                            if (!success) {
                                // 只在 source 请求时记录错误
                                if (errorStats != null) {
                                    errorStats.add(false);
                                }
                                return Mono.error(new RuntimeException(
                                        String.format("Source request body status not 200: %s, body: %s",
                                                bodyStatus, body)));
                            }
                            // 只在 source 请求时记录错误
                            if (errorStats != null) {
                                errorStats.add(true);
                            }

                            exchange.getResponse().setStatusCode(responseStatus);
                            exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());
                            DataBuffer buffer = exchange.getResponse().bufferFactory()
                                    .wrap(body.getBytes(StandardCharsets.UTF_8));
                            return exchange.getResponse().writeWith(Mono.just(buffer));

                        } catch (Exception e) {
                            log.error("Error parsing response body: {}", e.getMessage());
                            return Mono.error(new RuntimeException(
                                    String.format("Failed to parse response body as JSON: %s", body)));
                        }
                    });
        }

        exchange.getResponse().setStatusCode(responseStatus);
        exchange.getResponse().getHeaders().putAll(clientResponse.headers().asHttpHeaders());

        return clientResponse.bodyToMono(DataBuffer.class)
                .flatMap(buffer -> exchange.getResponse().writeWith(Mono.just(buffer)))
                .onErrorResume(throwable -> {
                    log.info("Error handling response: {}", throwable.getMessage());
                    exchange.getResponse().setStatusCode(HttpStatus.INTERNAL_SERVER_ERROR);
                    String errorMessage = throwable.getMessage();
                    DataBuffer buffer = exchange.getResponse().bufferFactory()
                            .wrap(errorMessage.getBytes(StandardCharsets.UTF_8));
                    return exchange.getResponse().writeWith(Mono.just(buffer));
                });
    }

    private boolean requiresRequestBody(HttpMethod method) {
        return method == HttpMethod.POST ||
                method == HttpMethod.PUT ||
                method == HttpMethod.PATCH;
    }

    @Data
    public static class Config {
    }
}